import numpy as np
import pandas as pd


class SectorNakamoto(object):
    def __init__(self, data):
        self.data = data
        self.nakamoto_coefficient = self.generate_nakamoto_coefficient()
    
    def generate_nakamoto_coefficient(self):
        cutoff = 0.51
        nakamoto_test = self.data > cutoff 
        nakamoto = sum(nakamoto_test)
        return nakamoto

    def get_nakamoto_coefficient(self):
        if self.nakamoto_coefficient:
            return self.nakamoto_coefficient


class Nakamoto(object):
    def __init__(self, nakamoto_list):
        self.nakamoto_dict = {}
        self.gini_dict = {}
        self.nakamoto_list = nakamoto_list
        self.minimum_nakamoto_id = None
        self.maximum_gini_id = None
        for nakamoto_obj in self.nakamoto_list:
            self.nakamoto_dict[nakamoto_obj.type] = nakamoto_obj.get_nakamoto_coefficient()
            self.gini_dict[nakamoto_obj.type] = nakamoto_obj.get_gini_coefficient()
        self.minimum_nakamoto = None 
        self.maximum_gini = None

    def generate_minimum_nakamoto(self):
        self.minimum_nakamoto_id = min(self.nakamoto_dict, key=self.nakamoto_dict.get)
        minimum_nakamoto_value = self.nakamoto_dict[self.minimum_nakamoto_id]
        return minimum_nakamoto_value

    def generate_maximum_gini(self):
        self.maximum_gini_id = max(self.gini_dict, key=self.gini_dict.get)
        maximum_gini_value = self.gini_dict[self.maximum_gini_id]
        return maximum_gini_value

    def get_minimum_nakamoto(self):
        if not self.minimum_nakamoto:
            self.minimum_nakamoto = self.generate_minimum_nakamoto()
        return self.minimum_nakamoto

    def get_maximum_gini(self):
        if not self.maximum_gini:
            self.maximum_gini = self.generate_maximum_gini()
        return self.maximum_gini

    def get_minimum_nakamoto_id(self):
        if not self.minimum_nakamoto_id:
            _ = self.generate_minimum_nakamoto()
        return self.minimum_nakamoto_id

    def get_maximum_gini_id(self):
        if not self.maximum_gini_id:
            _ = self.generate_maximum_gini()
        return self.maximum_gini_id
    
    def get_sector_series(self, sector):
        data_labels = ['Gini Coefficient', 'Nakamoto Coefficient']
        data = [round(sector.get_gini_coefficient(), 3), 
                sector.get_nakamoto_coefficient()]
        series = pd.Series(data, index=data_labels, name=sector.type)
        return series

    def get_summary(self):
        sector_series = []
        for sector in self.nakamoto_list:
            series = self.get_sector_series(sector)
            sector_series.append(series)
        nakamoto_dataframe = pd.concat(sector_series, axis=1)
        return nakamoto_dataframe.transpose()



from .sector import Sector
import requests
import json
import pandas as pd
import numpy as np


class Geography(Sector):
    def __init__(self, currency, **kwargs):
        super(Geography, self).__init__(currency, **kwargs)
        self.type = 'geography'
        ether_test = (self.currency == 'ETC') or (self.currency == 'ETH') 
        if ether_test:
            self.generate_evm_geo_data()
        else:
            raise "Node Geography only implemented for EVM-based Nodes"

    def generate_evm_geo_data(self):
        """
            This method aggregates ALL EVM data, not separating them by chain ID.
            So, ETH and ETC data are combined. TODO: Side project to gather this data.
        """
		
		## This URL is the API call ethernodes.org makes to get the node data.
        url = """
		https://www.ethernodes.org/network/1/data?draw=1&columns[0][data]=id&columns[0][name]=&columns[0][searchable]=true&columns[0][orderable]=true&columns[0][search][value]=&columns[0][search][regex]=false&columns[1][data]=host&columns[1][name]=&columns[1][searchable]=true&columns[1][orderable]=true&columns[1][search][value]=&columns[1][search][regex]=false&columns[2][data]=port&columns[2][name]=&columns[2][searchable]=true&columns[2][orderable]=true&columns[2][search][value]=&columns[2][search][regex]=false&columns[3][data]=country&columns[3][name]=&columns[3][searchable]=true&columns[3][orderable]=true&columns[3][search][value]=&columns[3][search][regex]=false&columns[4][data]=clientId&columns[4][name]=&columns[4][searchable]=true&columns[4][orderable]=true&columns[4][search][value]=&columns[4][search][regex]=false&columns[5][data]=client&columns[5][name]=&columns[5][searchable]=true&columns[5][orderable]=true&columns[5][search][value]=&columns[5][search][regex]=false&columns[6][data]=clientVersion&columns[6][name]=&columns[6][searchable]=true&columns[6][orderable]=true&columns[6][search][value]=&columns[6][search][regex]=false&columns[7][data]=os&columns[7][name]=&columns[7][searchable]=true&columns[7][orderable]=true&columns[7][search][value]=&columns[7][search][regex]=false&columns[8][data]=lastUpdate&columns[8][name]=&columns[8][searchable]=true&columns[8][orderable]=true&columns[8][search][value]=&columns[8][search][regex]=false&order[0][column]=0&order[0][dir]=asc&start=0&length=1000000&search[value]=&search[regex]=false&_=1545071939943 
      	"""
        
        response = requests.get(url)
        response_hash = json.loads(response.text).get('data')
        df = pd.DataFrame.from_dict(response_hash)
        country_raw = df.groupby('country').nunique()
        country = country_raw['id']
        country = country.sort_values()
        self.data = np.array(country)



from .analysis import Gini, LorenzPlot
from nakamoto.coefficient import SectorNakamoto
import uuid
import numpy as np


class Sector(object):
    def __init__(self, currency, **kwargs):
        self.uuid = uuid.uuid4()
        self.data = None 
        self.type = None
        self.plot_notebook = kwargs.get('plot_notebook')
        self.plot_image_path = kwargs.get('plot_image_path') 
        self.currency = currency
        self.lorenz_data = None
        self.lorenz_object = None
        self.nakamoto = None
        self.title = None
        self.gini = None

    def generate_gini_coefficient(self):
        if self.data is not None:
            gini_object = Gini(self.data)
            gini = gini_object.get_gini()
            return gini
        else:
            raise Exception('Cannot generate gini. No data')
    
    def generate_nakamoto_coefficient(self):
        if not self.lorenz_data:
            self.lorenz_data = self.generate_lorenz_data()
        nakamoto_object = SectorNakamoto(self.lorenz_data)
        nakamoto = nakamoto_object.get_nakamoto_coefficient()
        return nakamoto

    def generate_lorenz_object(self):
        file_name = f'{self.currency}_{self.type}_gini_{self.uuid}'
        title_name = f'{self.currency} {self.type.capitalize()} Lorenz Curve'
        lorenz_object = LorenzPlot(self.plot_notebook, self.plot_image_path, self.data, file_name, title_name)
        return lorenz_object

    def generate_lorenz_data(self):
        if not self.lorenz_object:
            self.lorenz_object = self.generate_lorenz_object()
        lorenz_data = self.lorenz_object.get_lorenz_data()
        return lorenz_data

    def generate_lorenz_curve(self):
        if not self.lorenz_object:
            self.lorenz_object = self.generate_lorenz_object()
        self.lorenz_object.get_plot()

    def get_lorenz_data(self):
        if not self.lorenz_data:
            self.lorenz_data = self.generate_lorenz_data()
        return self.lorenz_data

    def get_gini_coefficient(self):
        if not self.gini:
            self.gini = self.generate_gini_coefficient()
        return self.gini

    def get_plot(self):
        self.generate_lorenz_curve()

    def get_nakamoto_coefficient(self):
        if not self.nakamoto:
            self.nakamoto = self.generate_nakamoto_coefficient()
        return self.nakamoto

    
class CustomSector(object):
    def __init__(self, data, currency, sector_type, **kwargs):
        self.uuid = uuid.uuid4()
        self.data = data
        if type(self.data) is not np.ndarray:
            raise Exception('Sector data must be a numpy array')
        if len(self.data) == 0:
            raise Exception('Cannot pass empty data numpy array')
        self.plot_notebook = kwargs.get('plot_notebook')
        self.plot_image_path = kwargs.get('plot_image_path') 
        self.gini = None 
        self.currency = currency
        self.type = sector_type
        self.nakamoto = None 
        self.lorenz_data = None
        self.lorenz_object = None

    def generate_gini_coefficient(self):
        if self.data is not None:
            gini_object = Gini(self.data)
            gini = gini_object.get_gini()
            return gini
        else:
            raise Exception('Cannot generate gini. No data')
    
    def generate_nakamoto_coefficient(self):
        if not self.lorenz_data:
            self.lorenz_data = self.generate_lorenz_data()
        nakamoto_object = SectorNakamoto(self.lorenz_data)
        nakamoto = nakamoto_object.get_nakamoto_coefficient()
        return nakamoto

    def generate_lorenz_object(self):
        file_name = f'{self.currency}_{self.type}_gini_{self.uuid}'
        title_name = f'{self.currency} {self.type.capitalize()} Lorenz Curve'
        lorenz_object = LorenzPlot(self.plot_notebook, self.plot_image_path, self.data, file_name, title_name)
        return lorenz_object

    def generate_lorenz_data(self):
        if not self.lorenz_object:
            self.lorenz_object = self.generate_lorenz_object()
        lorenz_data = self.lorenz_object.get_lorenz_data()
        return lorenz_data

    def generate_lorenz_curve(self):
        if not self.lorenz_object:
            self.lorenz_object = self.generate_lorenz_object()
        self.lorenz_object.get_plot()

    def get_lorenz_data(self):
        if not self.lorenz_data:
            self.lorenz_data = self.generate_lorenz_data()
        return self.lorenz_data

    def get_gini_coefficient(self):
        if not self.gini:
            self.gini = self.generate_gini_coefficient()
        return self.gini

    def get_plot(self):
        self.generate_lorenz_curve()

    def get_nakamoto_coefficient(self):
        if not self.nakamoto:
            self.nakamoto = self.generate_nakamoto_coefficient()
        return self.nakamoto
